/* Tree to/from string, and other tree manipulation functions for procov.
 * Also init_bl code for setting branch lengths from a distance matrix
 * Originally based on NHML
 */

#include "mynhmlg.h"

void pc_treeprint(const noeud p)
{
  if (p->v1) pc_treeprint(p->v1);

  fprintf(stderr, "%s: %12s:%d:%.2g;\t%12s:%d:%.2g;\t%12s:%d:%.2g\n", p->nom,
	  (p->v1)?p->v1->nom:"v1", p->alive1, p->l1,
	  (p->v2)?p->v2->nom:"v2", p->alive2, p->l2,
	  (p->v3)?p->v3->nom:"v3", p->alive3, p->l3 );

  if (p->v2) pc_treeprint(p->v2);
}


void aj(int *liste, int nb) { // push
  while (*liste) liste++;
  *liste = nb;
}


char* readtree(FILE* treef, int nbsp){

  char *line;
  char *ctree, *ctreefin, *prov;
  int i;

  line=(char*)check_alloc(MAXLLINE+1, sizeof(char));
  ctree=check_alloc(3*MAXLNAME*nbsp, sizeof(char));
  ctreefin=ctree;
  
  line[0]='\n';
  while(line[0]=='\n') fgets(line, MAXLLINE, treef);
  i=0;
  while(line[i]==' ' || line[i]=='\t') i++;
  if(line[i]!='(' && line[i]!='['){
    printf("Tree first character should be '[' or '('\n");
    return NULL;
  }
  if(line[i]=='['){
    while((prov=strchr(line, ']'))==NULL){
      if(!fgets(line, MAXLLINE, treef)){
	printf("Unterminated comment in tree ('[' missing)\n");
        return NULL;
      }
    }
    prov++;
    while(*prov==' ' || *prov=='\t') prov++;
    if(*prov=='\n'){
      line[0]='\n';
      while(line[0]=='\n' && fgets(line, MAXLLINE, treef));
      prov=line;
      while(*prov==' ' || *prov=='\t') prov++;
    }
    if(*prov!='('){
      printf("Comments should be followed by '('\n");
      return NULL;
    }
  }
  else prov=line+i;	/* prov = premiere parenthese */


  sprintf(ctree, "%s", prov);
  while(*ctreefin!='\n' && *ctreefin!=';' && *ctreefin) ctreefin++;
  if(*ctreefin==';') return ctree;
  while(fgets(line, MAXLLINE, treef)){
    sprintf(ctreefin, "%s", line);
    if( (prov=strchr(line, ';')) ){
      *(prov+1)=0;
      return ctree;
    }
    while(*ctreefin!='\n' && *ctreefin) ctreefin++;  
  }
  *ctreefin=0;

  free(line);
  return ctree;
}



/* ctot */
/* ChaineTOTable. Read c_tree input (string) and write t_tree arbre (int**), */
/* branch length lgbi (internal) and lgbp (terminal), bootstrap values, */
/* species names (nom) and rooted/unrooted (racine-> r (rooted) or n (not). */
/* (Aquifex:0.1, (Thermotoga:0.1, (Thermus:0.1, (Rickettsia:0.1, Deinococcus:0.1)50:0.1)75:0.1):0.1); with bootstrap value */
/* (Aquifex:0.1, (Thermotoga:0.1, (Rickettsia:0.1, (Thermus:0.1, Deinococcus:0.1):0.1):0.1):0.1); without bootrap value */

int ctot(char *input, int *arbre[], double *lgbi, double *lgbp, double* bootstrap, char *nom[], char *racine, int* nbbi){


  int   i=0, j, k, fdf=0, fin=0, nbpo=0,
	nbpf = 0, cptv1 = 0, nbotu, br_ouverte, *listecour,
	otu = -1, pomoinspf, cpttree=0, t=1, nbnom=0;
  char  c, cc, cas, dejalu = '\0';
  double  f;

  sscanf(input+(cpttree++), "%c", &c);
  if (c == '[') {
    while ((c != ']') && (fdf != EOF)) fdf = sscanf(input+(cpttree++), "%c", &c);
    if (c != ']'){
      printf("Unmatched '[' ']'\n");
      return -1;
    }
  } 
  else
    if (c == '(') cpttree=0;
    else{
      printf("Tree file 1st character must be '(' or '['\n");
      return -1;
    }
  while ((c != ';') && (fdf != EOF)) {
    fdf = sscanf(input+(cpttree++), "%c", &c);
      if (c == '(') nbpo++;
      if (c == ')') nbpf++;
      if ((nbpo == nbpf + 1) && (c == ',')) cptv1++;
  }

  if (c != ';'){
    printf("';' missing at end of tree file\n");
    return -1;
  }

  if (nbpo != nbpf){
    printf("Unmatched parenthesis\n");
    return -1;
  }

  if (cptv1 == 1) cas = 'c';
  if (cptv1 == 2) cas = 'a';

  if ((cptv1!=1) && (cptv1!=2)){
    printf("Bad number of ',' in tree file\n");
    return -1;
  }

  nbotu = nbpo + 2;


  listecour=(int*) check_alloc(nbotu, sizeof(int));

  if (cas == 'a') *racine='n'; else *racine='r';

  if (cas=='c'){
    if (lgbp) lgbp[0] = 0.;
    strcpy(nom[0],"ROOT");
    for(i=0;i<nbotu-3;i++) arbre[0][i]=0;
    otu++;
  }


  cpttree=0;
  sscanf(input+(cpttree++), "%c", &c);
  if (c == '['){
    while (c != ']') sscanf(input+(cpttree++), "%c", &c);
    while((c==']') || (c==' ') || (c=='\n') || (c=='\t')) sscanf(input+(cpttree++), "%c", &c);
    if (c!='(') return -1;
  }
  else
    while(c!='(') sscanf(input+(cpttree++), "%c", &c);

  pomoinspf=1;
  for (i = 0; i < nbotu; i++) listecour[i] = 0;
  br_ouverte = 0;


  for (k = 0; t==1; k++) {
    if (dejalu == '\0') sscanf(input+(cpttree++), "%c", &c);
    else {
      c = dejalu;
      dejalu = '\0';
    }
    switch (c) {
    case ';': fin = 1; break;
    case ',': case '\n': case '\t': case '\'': case ' ': break;
    case '(':
      pomoinspf ++;
      br_ouverte++;
      aj(listecour, br_ouverte); break;
    case ')':
      pomoinspf--;
      sscanf(input+(cpttree++), "%c", &cc);
      if (cc == ';' || pomoinspf==0) {
	fin = 1; break;
      }
      j = retder(listecour);
      while (cc=='\n' || cc==' ' || cc=='\t') sscanf(input+(cpttree++),"%c",&cc);
      if (strpbrk(input+cpttree-1, "-0123456789.Ee")==input+cpttree-1){
        sscanf(input+cpttree-1, "%le", bootstrap+j-1);
        cpttree+=strspn(input+cpttree-1,"-.0123456789Ee");
        cc=*(input+cpttree-1);
        while (cc=='\n' || cc==' ' || cc=='\t') sscanf(input+(cpttree++),"%c",&cc);
      }
      if (cc == ':') {
	while(input[cpttree]==' ') cpttree++;
	sscanf(input+cpttree, "%le", &f);
	cpttree+=strspn(input+cpttree,"-0123456789.Ee");
	lgbi[j - 1] = f;
      }
      else dejalu=cc;
      break;
    default:
      nbnom++;
      otu++; cc = c; i = 0;
      while ((cc != ':') && (cc != ',') && (cc != ')') && (cc != '\n') && (cc!=' ')) {
	if (cc != '\'') { nom[otu][i] = cc; i++; nom[otu][i]='\0'; }
	sscanf(input+cpttree, "%c", &cc);
        cpttree++;
      }
      while ((cc != ':') && (cc != ',') && (cc != ')') && (cc != '\n')){
	sscanf(input+cpttree, "%c", &cc);
        cpttree++;
      }
      if (cc == ':') {
	while(input[cpttree]==' ') cpttree++;
	sscanf(input+(cpttree), "%le", &f);
	cpttree+=strspn(input+cpttree,"-0123456789.Ee");
	lgbp[otu] = f;
      } 
      else dejalu = cc;
      for (i = 0; i < nbotu - 3; i++) arbre[otu][i] = 0;
      for (i = 0; i < nbotu; i++)
	if (listecour[i] != 0)
	  arbre[otu][listecour[i] - 1] = 1;
    }
    if (fin == 1) break;
  }

  if(nbbi) *nbbi=br_ouverte;

  free(listecour);

  return nbnom;

/*  if (cas=='a') return nbotu; else return (nbotu-1);*/
}



/* ctreewrite */
/* Write string describing the s_tree underlying to noeud node at address ctree. */
static char* ctreewrite(noeud comesfrom, noeud node, char* ctree){
  noeud vd, vg;
  double l;

  if(comesfrom==node->v1)      { vg=node->v2; vd=node->v3; l=node->l1;}
  else if(comesfrom==node->v2) { vg=node->v1; vd=node->v3; l=node->l2;}
  else			       { vg=node->v1; vd=node->v2; l=node->l3;}
  if (vg==NULL){
    strcpy(ctree, node->nom);
    while(*ctree) ctree++;
    //    if (b>-0.1) sprintf(ctree," %.2f",b);
    while(*ctree) ctree++;
    if (l>-0.5) sprintf(ctree,":%.4f",l);
    while(*ctree) ctree++;
    return ctree;
  }
  else{
    *(ctree++)='(';
    ctree=ctreewrite(node, vg, ctree);
  }
  strcpy(ctree,", ");
  ctree += 2;
  ctree = ctreewrite(node, vd, ctree);
  *(ctree++) = ')';
  //  if (b>-0.1) sprintf(ctree,"%.2f",b);
  while(*ctree) ctree++;
  if(l>-0.9) sprintf(ctree,":%.6f", l);
  while(*ctree) ctree++;
  return ctree;
}


/* stoc */
/* StructTOChaine. Write string describing s_tree arbre_s at address ctree */
/* racine = unrooted/rooted (0/other) , notu = number of leaves. */
void stoc(noeud* arbre_s, int racine, int notu, char* ctree)
{
  char *tring, *prov = ctree;
  while(*prov) {*prov=0; prov++;}
  // tring=(char*)check_alloc(50*notu,sizeof(char));

  if(racine){
    tring=ctreewrite(NULL, root, ctree);
    prov=ctree+strlen(ctree)-1;
    while(*prov!=')') *(prov--)='\0';
    strcat(ctree,";");
  }else{
    tring=ctreewrite(arbre_s[2*notu-3]->v3, arbre_s[2*notu-3], ctree);
    prov=ctree+strlen(ctree)-1;
    while(*prov!=')') *(prov--)='\0';
    *(prov++)=',';
    *(prov++)=' ';
    tring=ctreewrite(arbre_s[2*notu-3], arbre_s[2*notu-3]->v3, prov);
    strcat(ctree,");");
  }
}

// dump tree for debugging
void output_tree_comp(noeud nd, FILE* out, tree* s_tree)
{
  static int numappel=0;
  int indbl;
  float m1=-1.;
  double truelg;

  if(nd==root){
    numappel++;
    if(out) fprintf(out, ";;data set %d\n", numappel);
    else return;
    //    set_node_deduced_comp(root);
  }

/* trouver le numero de parametre de la branche et du GC du noeud */

  indbl=s_tree->which_var_param[1]+nd->numerobr;

  /* which_var_param[4] in the following 2 lines of code cannot be computed:
subscript out of range. param[4] is for GC content which is not defined in
the protein model */
  /*  indgc=s_tree->which_var_param[4]+nd->numerobr;
  if(nd==root->v2) indgc=s_tree->which_var_param[4]+2*s_tree->nbseq-3; */

  //truelg=true_length(nd);
  truelg = nd->l3;

  if(nd->v1==NULL){
    fprintf(out, "noeud %s (depth 0): bl=%f, truel=%f, dlkh/dbl=%f, d2lkh/dbl2=%f\n" /* dlkh/dgc=%f, d2lkh/dgc2=%f, gc=%f\n" */ ,
	    nd->nom, nd->l3, truelg, (nd==root || indbl<0)?m1:s_tree->deriv[indbl], (nd==root|| indbl<0)?m1:s_tree->deriv2[indbl] /*, (nd==root || indgc<0)?m1:s_tree->deriv[indgc], (nd==root || indgc<0)?m1:s_tree->deriv2[indgc], nd->c+nd->g */);
    return;
  }

  fprintf(out, "noeud %s (v1=%s , v2=%s) : bl=%f, truel=%f, dlkh/dbl=%f, d2lkh/dbl2=%f\n" /* , dlkh/dgc=%f, d2lkh/dgc2=%f, gc=%f\n" */, nd->nom, nd->v1->nom, nd->v2->nom, nd->l3, truelg, (nd==root || indbl<0)?m1:s_tree->deriv[indbl], (nd==root || indbl<0)?m1:s_tree->deriv2[indbl] /*, (nd==root || indgc<0)?m1:s_tree->deriv[indgc], (nd==root || indgc<0)?m1:s_tree->deriv2[indgc], nd->c+nd->g */ );

  output_tree_comp(nd->v1, out, s_tree);
  output_tree_comp(nd->v2, out, s_tree);
}




// simple stack, end of the array is the top of the stack
int retder(int *liste){ // pop
  while (*(liste+1)) liste++;
  int tmp = *liste;
  *liste = 0;
  return tmp;
}


static void bit1(int *plist, int num){
  num--;
  plist += (num/lmot);
  *plist |=  (1<<(num%lmot));
}
static void bit0(int *plist, int num){
  num--;
  plist += (num/lmot);
  *plist &= ~(1<<(num%lmot));
}
int testbit(int *plist, int num){
  num--;
  plist += (num/lmot);
  return (*plist) & (1<<(num%lmot));
}


/* ctob */
/* ChaineTOBits. Read c_tree carbre and write b_tree barbre. */
/* only used by init_bl */
int ctob(char *carbre, char *nom[], int *barbre[])
{
  int   i=0, j, k, fin=0, nbpo=0,
	nbpf = 0, cptv1 = 0, nbotu, br_ouverte, *listecour,
	otu = -1, pomoinspf, cpttree=0, t=1;
  char  c, cc, dejalu = '\0', readname[30];

  c = carbre[cpttree++];
  if (c == '[') {
    while ((c != ']') && c) c = carbre[cpttree++];
    if (c != ']'){
      fputs("procov: tree error: Unmatched '[' ']'\n", stderr);
      return -1;
    }
  }
  else
    if (c == '(') cpttree=0;
    else{
      fputs("procov: error: Tree file 1st character must be '(' or '['\n", stderr);
      return -1;
    }
  while ((c != ';') && c ) {
    c = carbre[cpttree++];
    if (c == '(') nbpo++;
    if (c == ')') nbpf++;
    if ((nbpo == nbpf + 1) && (c == ',')) cptv1++;
  }

  if (c != ';'){
    fputs("procov: error: ';' missing at end of tree\n", stderr);
    return -1;
  }

  if (nbpo != nbpf){
    fputs("procov: error: Unmatched parentheses in tree\n", stderr);
    return -1;
  }

  if (cptv1 == 2) /* unrooted : ok */;
  else
    if (cptv1 == 1){ /* rooted : problem */
      fputs("procov: error: Unexpected rooted tree.\n", stderr);
      return -1;
    }
    else{ /* bad tree string */
      fputs("procov: error: Bad number of ',' in tree\n", stderr);
      return -1;
    }

  nbotu=nbpo+2;


  listecour=(int*) check_alloc(nbotu, sizeof(int));

  cpttree=0;
  c = carbre[cpttree++];
  if (c == '['){
    while (c != ']') c = carbre[cpttree++];
    while((c==']') || (c==' ') || (c=='\n') || (c=='\t')) c = carbre[cpttree++];
    if (c!='(') return -1;
  }
  else
    while(c!='(') c = carbre[cpttree++];

  pomoinspf=1;
  for (i = 0; i < nbotu; i++) listecour[i] = 0;
  br_ouverte = 0;


  for (k = 0; t==1; k++) {
    if (dejalu == '\0') c = carbre[cpttree++];
    else {
      c = dejalu;
      dejalu = '\0';
    }
    switch (c) {
    case ';': fin = 1; break;
    case ',': case '\n': case '\t': case '\'': case ' ': break;
    case '(':
      pomoinspf ++;
      br_ouverte++;
      aj(listecour, br_ouverte); break;
    case ')':
      pomoinspf--;
      if (*(carbre+cpttree+1) == ';' || pomoinspf==0) {
	fin = 1; break;
      }
      j = retder(listecour);
      while(carbre[cpttree]!=',' && carbre[cpttree]!=')') cpttree++;
      break;
    default:
      otu=-1; cc = c; i = 0;
      while ((cc != ':') && (cc != ',') && (cc != ')') && (cc != '\n') && (cc != ' ')) {
	if (cc != '\'') { readname[i++] = cc; readname[i]='\0'; }
	cc = carbre[cpttree++];
      }
      for(j=0;j<nbotu;j++) if(samename(readname, nom[j])) { otu=j; break; }
      if (otu==-1) { fprintf(stderr, "procov: error: Unknown name : %s\n", readname); return -1; }
      for(i=0;i<nbotu-3;i++) bit0(barbre[i], otu+1);
      for (i = 0; i < nbotu-3; i++)
	if (listecour[i] != 0)
	  bit1(barbre[listecour[i]-1], otu+1);
      cpttree--;
      while(carbre[cpttree]!=',' && carbre[cpttree]!=')') cpttree++;
    }
    if (fin == 1) break;
  }

  free(listecour);
  return nbotu;
}





/* ttos */
/* TableTOStruct. Create s_tree arbre_s from : t_tree arbre_t, leaves number notu, */
/* branch lengths lgbi (internal branches) , lgbp (terminal branches). */
/* If tree has no branch length, arguments 3 and 4 must be NULL. */
static int ttos(int** arbre_t, int notu, double* lgbi, double* lgbp, double* alive, char** nom, noeud* arbre_s)
{
  noeud   	p1, p2;
  int           notuv, i, j, k, sommebi, tax1, tax2, n1, n2, n3, cpt1=-1;
  int           *kill_tax, *kill_bi;
  double	arg6, arg9;

  sommebi=n1=n2=n3=0; // keep gcc happy
  kill_tax = (int *)check_alloc(notu, sizeof(int));
  kill_bi = (int *)check_alloc(notu, sizeof(int));

  notuv = notu;
  for (i = 0; i <notu; i++){
    kill_tax[i] = 0;
    kill_bi[i] = 0;
  }

	/* terminal nodes */

  for (i = 0; i < notu; i++){
    if(lgbp==NULL) arg6=-1.0; else arg6=lgbp[i];
    arbre_s[i] = create_node(NULL, NULL, NULL, 0., 0., arg6, -1, -1, 1, nom[i]);
  }


	/* internal nodes */

  for (i = 0; i < notu - 3; i++) {
	/* determination de la bi a creer */
    for (j = 0; j < notu - 3 ; j++) {
      if (kill_bi[j] == 0) {
	sommebi = 0;
	for (k = 0; k < notu; k++)
	  if (kill_tax[k] == 0) {
	    sommebi += arbre_t[k][j];
	  }
	if (sommebi == 2 || notuv - sommebi == 2)
	  break;
      }
    }

	/* determination des 2 otus/noeuds fils */
    tax1=tax2=0; // just keep gcc happy
    if (sommebi == 2) {
      for (k = 0; k < notu; k++)
	if (arbre_t[k][j] == 1 && kill_tax[k] == 0) {
	  tax1 = k;
	  break;
	}
      for (k = tax1 + 1; k < notu; k++)
	if (arbre_t[k][j] == 1 && kill_tax[k] == 0) {
	  tax2 = k;
	  break;
	}
    } else {
      for (k = 0; k < notu; k++)
	if (arbre_t[k][j] == 0 && kill_tax[k] == 0) {
	  tax1 = k;
	  break;
	}
      for (k = tax1 + 1; k < notu; k++)
	if (arbre_t[k][j] == 0 && kill_tax[k] == 0) {
	  tax2 = k;
	  break;
	}
    }

    p1 = findroot(arbre_s[tax1], 2*notu);
    p2 = findroot(arbre_s[tax2], 2*notu);

    if (lgbp) arg6=lgbi[j]; else arg6=0.;
    if (alive) arg9=alive[j]; else arg9=-1.;

    arbre_s[notu + (++cpt1)] = create_node(p1, p2, NULL, p1->l3, p2->l3, arg6, p1->alive3, p2->alive3, arg9, NULL);


    p1->v3 = arbre_s[notu + cpt1];
    p2->v3 = arbre_s[notu + cpt1];

    kill_tax[tax1] = kill_bi[j] = 1;
    notuv--;
  }


	/* last node */
  for (i = 0; i < 2 * notu - 3; i++)
    if (arbre_s[i]->v3 == NULL) {
      n1 = i;
      break;
    }

  for (i = n1 + 1; i < 2 * notu - 3; i++)
    if (arbre_s[i]->v3 == NULL) {
      n2 = i;
      break;
    }

  for (i = n2 + 1; i < 2 * notu - 3; i++)
    if (arbre_s[i]->v3 == NULL) {
      n3 = i;
      break;
    }

  arbre_s[2 * notu - 3] = create_node(arbre_s[n1], arbre_s[n2], arbre_s[n3], arbre_s[n1]->l3, arbre_s[n2]->l3, arbre_s[n3]->l3, arbre_s[n1]->alive3, arbre_s[n2]->alive3, arbre_s[n3]->alive3, NULL);
  arbre_s[n1]->v3 = arbre_s[2 * notu - 3];
  arbre_s[n2]->v3 = arbre_s[2 * notu - 3];
  arbre_s[n3]->v3 = arbre_s[2 * notu - 3];

  free(kill_tax);
  free(kill_bi);
  return 0;
}



void makelistbr_unrooted(noeud from, noeud nd, branche* br, int* alivebr,int nbbranch){
  noeud to1, to2;
  double lto1, lto2;
  int i=0, alive1, alive2;

  if(nd==root){
    if(from==root->v1) makelistbr_unrooted(root, nd->v2, br, alivebr, nbbranch);
    if(from==root->v2) makelistbr_unrooted(root, nd->v1, br, alivebr, nbbranch);
    return;
  }

  if     (from==nd->v1){to1=nd->v2; to2=nd->v3; lto1=nd->l2; lto2=nd->l3; alive1=nd->alive2; alive2=nd->alive3;}
  else if(from==nd->v2){to1=nd->v1; to2=nd->v3; lto1=nd->l1; lto2=nd->l3; alive1=nd->alive1; alive2=nd->alive3;}
  else if(from==nd->v3){to1=nd->v1; to2=nd->v2; lto1=nd->l1; lto2=nd->l2; alive1=nd->alive1; alive2=nd->alive2;}
  else{
    to1 = to2 = NULL;
    lto1=lto2 = alive1=alive2 = 0; // keep gcc happy
    fputs("procov: error making branch list.\n", stderr);
  }


  while(i<nbbranch && br[i]->bout1) i++;

  if(to1 && to1==root) {
    if(nd==root->v1){
      lto1+=root->l2;
      br[i]->bout1=nd; br[i]->bout2=root->v2; br[i]->length=lto1; i++;
    }else{
      lto1+=root->l1;
      br[i]->bout1=nd; br[i]->bout2=root->v1; br[i]->length=lto1; i++;
    }
    if(alivebr) alivebr[i-1]=1;
  }
  if(to2 && to2==root) {
    if(nd==root->v1){
      lto2+=root->l2;
      br[i]->bout1=nd; br[i]->bout2=root->v2; br[i]->length=lto2; i++;
    }else{
      lto2+=root->l1;
      br[i]->bout1=nd; br[i]->bout2=root->v1; br[i]->length=lto2; i++;
    }
    if(alivebr) alivebr[i-1]=1;
  }


  if(to1 && to1!=root) {
    br[i]->bout1=nd;
    br[i]->bout2=to1;
    br[i]->length=lto1;
    if(alivebr) alivebr[i]=alive1;
    i++;
  }
  if(to2 && to2!=root) {
    br[i]->bout1=nd;
    br[i]->bout2=to2;
    br[i]->length=lto2;
    if(alivebr) alivebr[i]=alive2;
  }

  if(to1) makelistbr_unrooted(nd, to1, br, alivebr, nbbranch);
  if(to2) makelistbr_unrooted(nd, to2, br, alivebr, nbbranch);
}

void root_s(branche br, double frac1)
{
  noeud nd1=br->bout1, nd2=br->bout2;
  double l=br->length;

  root->v1=nd1; root->v2=nd2; root->l1=l*frac1; root->l2=l*(1-frac1);
  root->alive1=root->alive2=1; root->alive3=-1;
  //nd1->remain=nd2; nd2->remain=nd1;
  //nd1->lremain=l; nd2->lremain=l;
  if (nd1->v1==nd2) { nd1->v1=root; nd1->l1=l*frac1; /*nd1->alremain=nd1->alive1;*/ nd1->alive1=1;}
  else if (nd1->v2==nd2) { nd1->v2=root; nd1->l2=l*frac1; /*nd1->alremain=nd1->alive2;*/ nd1->alive2=1; }
  else if (nd1->v3==nd2) { nd1->v3=root; nd1->l3=l*frac1; /*nd1->alremain=nd1->alive3;*/ nd1->alive3=1; }
  else fputs("procov: Error rooting.\n", stderr);
  if (nd2->v1==nd1) { nd2->v1=root; nd2->l1=l*(1-frac1); /*nd2->alremain=nd2->alive1;*/ nd2->alive1=1; }
  else if (nd2->v2==nd1) { nd2->v2=root; nd2->l2=l*(1-frac1); /*nd2->alremain=nd2->alive2;*/ nd2->alive2=1; }
  else if (nd2->v3==nd1) { nd2->v3=root; nd2->l3=l*(1-frac1); /*nd2->alremain=nd2->alive3;*/ nd2->alive3=1; }
  else fputs("procov: Error rooting.\n", stderr);
}

void organize_tree(noeud from, noeud nd){
  noeud prov;
  double lprov;
  int alprov;
  static int numappel=0;

  if(!nd->v1) return;
  if(nd!=root){
    numappel++;
    sprintf(nd->nom, "int%d", numappel);
  }
  if(from==nd->v1) {
    prov=nd->v3; nd->v3=nd->v1; nd->v1=prov;
    lprov=nd->l3; nd->l3=nd->l1; nd->l1=lprov;
    alprov=nd->alive3; nd->alive3=nd->alive1; nd->alive1=alprov;
  }
  if(from==nd->v2) {
    prov=nd->v3; nd->v3=nd->v2; nd->v2=prov;
    lprov=nd->l3; nd->l3=nd->l2; nd->l2=lprov;
    alprov=nd->alive3; nd->alive3=nd->alive2; nd->alive2=alprov;
  }
  organize_tree(nd, nd->v1);
  organize_tree(nd, nd->v2);

  if(nd==root) numappel=0;
}


/* unroot */
/* Unroot t_tree treer. All parameters are modified (branch lengths, names, ...). */
/* Root info may be kept in list1-list2 l1-l2 fracroot1 if l1 is non NULL and list1 and list2  */
/* are allocated as nbtaxa-sized char* arrays. */

int unroot(int** treer, int notu, double* lgbi, double* lgbp, double* bootvals, char** nom, char** list1, char** list2, int* l1, int* l2, double* fracroot1)
{
  int i, j, j1, j2, br_a_virer=-1, bp_a_modifier=-1, bi_a_modifier=-1, somme1, somme2, drapeau;
  double l=DBL_MAX;

  for(j=0;j<notu-2;j++) if(treer[0][j]!=0) return 1;
  for(j1=0;j1<notu-3;j1++){
    somme1=0;
    for(i=0;i<notu+1;i++) somme1 += treer[i][j1];
    if (somme1==notu-1){
      br_a_virer=j1;
      if(lgbp){
        l=lgbi[j1];
        for(i=1;i<notu+1;i++) if(treer[i][j1]==0) {bp_a_modifier=i; break;}
      }
      break;
    }
    for(j2=j1+1;j2<notu-2;j2++){
      somme2=0;
      for(i=0;i<notu+1;i++) somme2+=treer[i][j2];
      if (somme1==notu-somme2){
	drapeau=0;
	for(i=1;i<notu;i++) if(abs(treer[i][j1]-treer[i][j2])!=1) drapeau=1;
	if (drapeau==0){
          br_a_virer=j1;
          if(lgbi) { bi_a_modifier=j2; l=lgbi[j1]; }
	  break;
	}
      }

      if (br_a_virer!=-1) break;
    }
    if (br_a_virer!=-1) break;
  }


  if(l1){
    j1=j2=0;
    for(i=1;i<=notu;i++){
      if(treer[i][br_a_virer]==0) {list1[j1]=nom[i]; j1++;}
      else {list2[j2]=nom[i]; j2++;}
    }
    if(j1<1 || j2<1) fprintf(stderr, "procov: problem at %s:%d\n", __func__, __LINE__);
    *l1=j1; *l2=j2;
    if (bp_a_modifier!=-1) *fracroot1=lgbp[bp_a_modifier]/(lgbp[bp_a_modifier]+l);
    if (bi_a_modifier!=-1) *fracroot1=lgbi[bi_a_modifier]/(lgbi[bi_a_modifier]+l);
  }


  if (bp_a_modifier!=-1) lgbp[bp_a_modifier]+=l;
  if (bi_a_modifier!=-1) lgbi[bi_a_modifier]+=l;
  for(i=br_a_virer;i<notu-3;i++){
    if (lgbi) lgbi[i]=lgbi[i+1];
    if (bootvals) bootvals[i]=bootvals[i+1];
  }
  if(lgbp) for(i=0;i<notu;i++) lgbp[i]=lgbp[i+1];
  if(nom){
    free(nom[0]);
    for(i=0;i<notu;i++) nom[i]=nom[i+1];
    nom[notu]=check_alloc(MAXLNAME+1, sizeof(char));
  }

  for(i=1;i<notu+1;i++)
    for(j=0;j<notu-2;j++)
      treer[i-1][j]=treer[i][j];

  for(j=br_a_virer+1;j<notu-2;j++)
    for(i=0;i<notu;i++)
      treer[i][j-1]=treer[i][j];

  return 0;
}


/* equallist */
/* Check the list of taxa down a branch. */
static int equallist(noeud from, noeud nd, int nb, char** list)
{
  noeud n1, n2;
  int i, flag;

  if(from==nd->v1) { n1=nd->v2; n2=nd->v3; }
  else if(from==nd->v2) { n1=nd->v1; n2=nd->v3; }
  else if(from==nd->v3) { n1=nd->v1; n2=nd->v2; }
  else { n1=n2=NULL; fprintf(stderr, "procov: error at:%s\n", __func__); }

  if(!n1){
    flag=0;
    for(i=0;i<nb;i++){
      if(samename(nd->nom, list[i])) {flag=1; break; }
    }
    return flag;
  }

  return(equallist(nd, n1, nb, list) && equallist(nd, n2, nb, list));
}


/* isbranch */
/* Check wether lists of taxa name list1 and list2 (respectively l1 */
/* and l2 taxa) exactly match the lists of taxa laying left-side  */
/* and right-side branch br. */
int isbranch(branche br, int l1, char** list1, int l2, char** list2)
{
  if( equallist(br->bout1, br->bout2, l1, list1) && equallist(br->bout2, br->bout1, l2, list2)) return 2;
  if( equallist(br->bout2, br->bout1, l1, list1) && equallist(br->bout1, br->bout2, l2, list2)) return 1;
  return 0;
}


/* taxa_count */
/* Return the number of taxa in c_tree ctree. */

static int taxa_count(char* ctree)
{
  int nbpo=0, nbpf=0, nbv=0;
  char* prov = ctree;

  while(*prov!='[' && *prov!='(' && *prov) prov++;
  if(*prov=='\0'){
    fputs("procov: error Trees should begin with '[' or '('\n", stderr);
    return -1;
  }
  if(*prov=='['){
    while(*prov!=']') prov++;
    while(*prov!='(') prov++;
  }

/* debut arbre */

  while(*prov && *prov!=';'){
    if(*prov=='(') nbpo++;
    if(*prov==')') nbpf++;
    if(nbpo-nbpf==1 && *prov==',') nbv++;
    prov++;
  }

  if(nbv==1){ /* rooted */
    return (nbpo+1);
  }
  if(nbv==2){ /* unrooted */
    return (nbpo+2);
  }
  fputs("procov: error: Bad number of commas ',' in tree\n", stderr);
  return -1;
}


noeud* ctos(char* ctree, int* notu, int *tree_is_rooted)
{
  int i, l1, l2, nbbi;
  double fracroot1;
  char racine, *list1[MAXNSP], *list2[MAXNSP];
  int nb = taxa_count(ctree);
  fracroot1 = l1 = l2 = racine = 0;  // compiler can't tell they're initialized

  double *lgbp = check_alloc(nb+1, sizeof(double));
  double *lgbi = check_alloc(nb+1, sizeof(double));
  double *bootvals = check_alloc(nb+1, sizeof(double));
  /*list1=check_alloc(2*nb+2, sizeof(char*));
  list2=check_alloc(2*nb+2, sizeof(char*));*/
  char **nom = check_alloc(nb+1, sizeof(char*));
  int **ttree = check_alloc(nb+1, sizeof(int**));
  for(i=0;i<nb+1;i++){
    nom[i]=(char*)check_alloc(MAXLNAME+1, sizeof(char));
    ttree[i]=(int*)check_alloc(nb, sizeof(int));
  }
  noeud *stree = check_alloc(2*nb, sizeof(noeud));
  branche *br_list = check_alloc(2*nb, sizeof(branche));
  for(i=0;i<2*nb-2;i++){
    br_list[i] = check_alloc(1, sizeof(struct branche));
  }

  nb=ctot(ctree, ttree, lgbi, lgbp, bootvals, nom, &racine, &nbbi);

  if(racine=='r'){
    unroot(ttree, nb, lgbi, lgbp, bootvals, nom, list1, list2, &l1, &l2, &fracroot1);
  }
  ttos(ttree, nb, lgbi, lgbp, bootvals, nom, stree);
  makelistbr_unrooted(NULL, stree[0], br_list, NULL, 2*nb-3);
  root=create_node(NULL, NULL, NULL, -1., -1., -1., -1, -1, -1, "ROOT");

  if(racine=='r'){
    *tree_is_rooted=1;
    for(i=0;i<2*nb-3;i++){
      if(isbranch(br_list[i], l1, list1, l2, list2)==1){
 	root_s(br_list[i], fracroot1);
	break;
      }
      if(isbranch(br_list[i], l1, list1, l2, list2)==2){
 	root_s(br_list[i], 1.-fracroot1);
	break;
      }
    }
    if(i==2*nb-3) fprintf(stderr, "procov: rooting error at: %s\n", __func__);
  }
  else{
    *tree_is_rooted=0;
    root_s(br_list[0], 0.5);
  }
  organize_tree(NULL, root);
  *notu=nb;

  free(lgbp);
  free(lgbi);
  free(bootvals);

  for(i=0;i<nb;i++){
    free(ttree[i]); free(nom[i]);
  }
  free(ttree); free(nom);
  /* free(list1); free(list2); */

  for(i=0;i<2*nb-2;i++)
    free(br_list[i]);
  free(br_list);

  return stree;
}
